#
# spec file for package dealii
#
# Copyright (c) 2024 SUSE LLC
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global flavor %{nil}

%define __builder ninja
%define sover 9.6.1
%define shlibver %(echo %{sover} | tr "." "_")
%define srcname dealii

# PYTHON BINDINGS CAN"T BE BUILT [https://github.com/dealii/dealii/issues/10423]
%bcond_with python

%if 0%{?suse_version} == 1500 && 0%{?sle_version} == 150100
%bcond_with doc
%else
%bcond_without doc
%endif

%if 0%{?suse_version} < 1650
%define gcc_ver 9
%endif

# SECTION MPI DEFINITIONS
%if "%{flavor}" == "openmpi4"
%global mpi_flavor openmpi
%define mpi_vers 4
%endif

%if "%{flavor}" == "openmpi5"
%global mpi_flavor openmpi
%define mpi_vers 5
%endif

%{?mpi_flavor:%{bcond_without mpi}}%{!?mpi_flavor:%{bcond_with mpi}}
%{?with_mpi:%{!?mpi_flavor:error "No MPI family specified!"}}

%if %{with mpi}
%define my_prefix  %{_libdir}/mpi/gcc/%{mpi_flavor}%{?mpi_vers}
%define my_bindir  %{my_prefix}/bin
%define my_libdir  %{my_prefix}/%{_lib}
%define my_incdir  %{my_prefix}/include
%define my_datadir %{my_prefix}/share
%define my_suffix  -%{mpi_flavor}%{?mpi_vers}
%else
%define my_prefix  %{_prefix}
%define my_bindir  %{_bindir}
%define my_libdir  %{_libdir}
%define my_incdir  %{_includedir}
%define my_datadir %{_datadir}
%endif
# /SECTION MPI DEFINITIONS

# SECTION PKG NAMING DEFINITIONS
%define pname %{srcname}%{?my_suffix}
%define shlib libdeal_II%{shlibver}%{?my_suffix}
%define shlib_debug libdeal_II_g%{shlibver}%{?my_suffix}
# /SECTION

%define memlim 2500
# LTO CAUSES aarch64 BUILDS TO TIME OUT; IT ALSO REQUIRES ~4GB PER THREAD
%ifarch aarch64
%define _lto_cflags %{nil}
%define memlim 4000
%endif
%ifarch ppc64
%define memlim 3000
%endif

Name:           %{pname}
Version:        9.6.1
Release:        3.2
Summary:        A Finite Element Differential Equations Analysis Library
License:        LGPL-2.1-or-later
URL:            https://www.dealii.org/
Source:         https://github.com/dealii/dealii/releases/download/v%{version}/%{srcname}-%{version}.tar.gz
# NOTE: serial arpack-ng even if parpack is available (see gh#dealii/dealii#10197)
# PATCH-FEATURE-OPENSUSE dealii-disable-parallel-doc-build.patch badshah400@gmail.com -- Disable parallel doc build for reproducibility; upstream doxygen issue https://github.com/doxygen/doxygen/issues/11138
Patch:          dealii-disable-parallel-doc-build.patch
BuildRequires:  arpack-ng-devel
BuildRequires:  blas-devel
BuildRequires:  cmake
BuildRequires:  gcc%{?gcc_ver}-c++
BuildRequires:  gcc%{?gcc_ver}-fortran
BuildRequires:  gmsh
BuildRequires:  gmsh-devel
BuildRequires:  hdf5%{?my_suffix}-devel
BuildRequires:  kokkos-devel >= 4
BuildRequires:  lapack-devel
BuildRequires:  libboost_iostreams-devel
BuildRequires:  libboost_serialization-devel
BuildRequires:  libboost_system-devel
BuildRequires:  libboost_thread-devel
BuildRequires:  memory-constraints
BuildRequires:  metis-devel
BuildRequires:  muparser-devel
BuildRequires:  ninja
BuildRequires:  pkgconfig
BuildRequires:  python3-base
BuildRequires:  suitesparse-devel
BuildRequires:  tbb-devel
BuildRequires:  pkgconfig(bzip2)
BuildRequires:  pkgconfig(gsl)
BuildRequires:  pkgconfig(zlib)
%if %{with doc}
BuildRequires:  doxygen
BuildRequires:  fdupes
%endif
%if %{with mpi}
BuildRequires:  %{mpi_flavor}%{?mpi_vers}-config
BuildRequires:  %{mpi_flavor}%{?mpi_vers}-devel
BuildRequires:  parpack%{my_suffix}-devel
BuildRequires:  scalapack%{my_suffix}-devel
%endif
%if %{with python}
BuildRequires:  libboost_python3-devel
BuildRequires:  python3-devel
%endif
# Builds fail for i586 due to linking taking too long and being mistaken by OBS workers for "Job stuck"
ExcludeArch:    i586

%description
deal.II is a C++ program library targeted at the computational solution of
partial differential equations using adaptive finite elements.

%package -n %{shlib}
Summary:        A generic C++ finite element library

%description -n %{shlib}
deal.II is a C++ program library targeted at the computational solution of
partial differential equations using adaptive finite elements.

This package provides the shared library for deal.II.

%package -n deal_II%{?my_suffix}-devel
Summary:        Development files for %{name}
Requires:       %{shlib} = %{version}
Provides:       %{name}%{?my_suffix}-devel = %{version}

%description -n  deal_II%{?my_suffix}-devel
The %{name}-devel package contains libraries and header files for
developing applications that use %{name}.

%package -n python3-PyDealII
Summary:        Python bindings for %{name}

%description -n python3-PyDealII
This package provides the python3 bindings for dealii.

%prep
%autosetup -p1 -n %{srcname}-%{version}

%build
%if %{with mpi}
source %{my_bindir}/mpivars.sh
export CC=mpicc
export CXX=mpicxx
%else
export CC=gcc%{?gcc_ver:-%{gcc_ver}}
export CXX=g++%{?gcc_ver:-%{gcc_ver}}
%endif

export CFLAGS="%{optflags}"
export CXXFLAGS="%{optflags}"

%limit_build -m %{memlim}

# BUILD ONLY SUPPORTS "Release", "Debug", OR "DebugRelease" FOR CMAKE_BUILD_TYPE
# BUT THE "-g" FLAG TO ENABLE DEBUGINFO IS AUTOMATICALLY ADDED BY %%{optflags};
# SO SIMPLY USE CMAKE_BUILD_TYPE="Release"
%cmake -DCMAKE_INSTALL_PREFIX:PATH=%{my_prefix} \
       -DCMAKE_PREFIX_PATH:PATH=%{my_prefix} \
       -DCMAKE_BUILD_TYPE:STRING="Release" \
       -DCMAKE_INSTALL_RPATH_USE_LINK_PATH:BOOL=OFF \
       -DDEAL_II_COMPONENT_DOCUMENTATION:BOOL=%{?with_doc:ON}%{!?with_doc:OFF} \
       -DDEAL_II_WITH_THREADS:BOOL=ON \
       -DDEAL_II_WITH_MPI:BOOL=%{?with_mpi:ON}%{!?with_mpi:OFF} \
       -DDEAL_II_COMPONENT_PYTHON_BINDINGS:BOOL=%{?with_python:ON}%{!?with_python:OFF} \
       -DDEAL_II_DOCHTML_RELDIR:PATH=%{_docdir}/%{name} \
       -DDEAL_II_EXAMPLES_RELDIR:PATH=%{_docdir}/%{name} \
       -DDEAL_II_COMPILE_EXAMPLES:BOOL=OFF \
       -DDEAL_II_ALLOW_BUNDLED:BOOL=OFF

%cmake_build

%install
%cmake_install

# REMOVE UNNECESSARY FILES
rm %{buildroot}%{my_prefix}/*.md \
   %{buildroot}%{my_prefix}/detailed.log \
   %{buildroot}%{my_prefix}/summary.log

# CLEAR UP UNNECESSARY HASHBANGS
sed -i "1{/\/bin\/sh/d}" %{buildroot}%{my_datadir}/deal.II/scripts/get_*_tag.sh
sed -i "1{/\/usr\/bin\/env/d}" %{buildroot}%{my_datadir}/deal.II/scripts/indent.py

%if %{with doc}
%fdupes %{buildroot}%{_docdir}/%{name}/
%endif

%post -n %{shlib} -p /sbin/ldconfig
%postun -n %{shlib} -p /sbin/ldconfig

%files -n %{shlib}
%{my_libdir}/libdeal_II.so.%{sover}

%files -n deal_II%{?my_suffix}-devel
%license LICENSE.md
%doc AUTHORS.md README.md CONTRIBUTING.md
%doc %{_docdir}/%{name}
%{my_incdir}/deal.II/
%{my_libdir}/*.so
%{my_datadir}/deal.II/
%{my_libdir}/cmake/

%if %{with python}
%files -n python3-PyDealII
%license LICENSE.md
%{python3_sitelib}/PyDealII/
%endif

%changelog
* Sun Jan  5 2025 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Remove obsolete netcdf build dependency, netcdf bindings were
  removed with version 9.3.3.
- Minor spec file cleanups.
* Fri Dec 27 2024 Atri Bhattacharya <badshah400@gmail.com>
- Add dealii-disable-parallel-doc-build.patch: Disable parallel
  doc build for reproducibility; upstream doxygen issue
  (gh#doxygen/doxygen#11138).
* Sat Dec 21 2024 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 9.6.1:
  * Tpetra: Fix is_vector_space_vector check with complex values
    (gh#dealii/dealii#17558).
  * Create new string when demangling stacktraces
    (gh#dealii/dealii#17575).
  * Cherry-pick bug fixes (gh#dealii/dealii#17874).
  * MatrixFree/TensorProductPointKernels: fix compilation with
    Number = float (gh#dealii/dealii#17924).
* Wed Nov 13 2024 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 9.6.0:
  * Numerous changes, see
    <https://www.dealii.org/developer/doxygen/deal.II/changes_between_9_5_2_and_9_6_0.html>
- Use BuildRequires: muparser-devel instead of pkgconfig(muparser)
  to break degeneracy caused by the fake pkgconfig provides of
  muparser-32bit-devel (see gh#openSUSE/obs-build#1002).
- Require GCC-9 for Leap 15.6 or lower as enforced by upstream.
* Fri Feb  2 2024 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 9.5.2:
  * Allow compilation with PETSc but without MPI
    (gh#dealii/dealii#15788).
  * Fix compiling with PETSc with complex scalar type
    (gh#dealii/dealii#15856).
  * base/mpi.h: also instantiate for signed long long int
    (gh#dealii/dealii#16540).
  * CMake: fix compilation with bundled boost and modern clang
    (gh#dealii/dealii#16541).
  * DofTools: add missing codimension-one instantiation
    (gh#dealii/dealii#16539).
- Enable openmpi5 flavours.
* Tue Oct 24 2023 Nicolas Morey <nicolas.morey@suse.com>
- Drop support for obsolete openmpi[23]
- Prepare support for openmpi5
* Sat Jul 22 2023 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 9.5.1:
  - Guard NOX support in include/deal.II/numerics/nonlinear.h
    (gh#dealii/dealii#15708).
  - bundled/tbb: fix compilation with gcc-13
    (gh#dealii/dealii#15726).
- Changes from version 9.5.0:
  - Substantial updates and extensions to deal.II's interfaces to
    other libraries. This includes, in particular, the integration
    of Kokkos additions and updates to the PETSc and Trilinos
    interfaces.
  - Uniform handling of nonlinear solver packages and a uniform
    way of defining callbacks used by external libraries.
  - Advances in matrix-free infrastructure.
  - Advances in non-matching support.
  - New features related to linear algebra.
  - C++ language modernization.
  - Build-system modernization.
- New BuildRequires: kokkos-devel >= 4.
- Drop dealii-rpmlintrc: Filters for openmpi* pkgs longer needed.
* Tue Feb 14 2023 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 9.4.2:
  * A compilation issue with step-70 has been resolved.
  * CMake: prefer -pthread for posix thread support.
  * A type mismatch for suitesparse has been fixed that lead to
    compilation failures on certain platforms.
* Sun Dec  4 2022 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 9.4.1:
  * cmake: always export compile_commands.json in deal.II and user
    projects
  * doxygen: fix various errors in formulas
  * doxygen: fix SymmetricTensor friends
  * cmake: fix PETSc version detection
  * base: fix some VectorizedArrayTypes for non-default
    vectorization
  * gitignore: ignore clangd files and directories
  * change ConsensusAlgorithm deprecations to early deprecated
  * step-81: Mention example step in the tutorial lists
  * bugfix: use correct tolerance in MappingCartesian check
* Sun Jul 10 2022 Atri Bhattacharya <badshah400@gmail.com>
- Use dealii-rpmlintrc file to suppress rpmlint false positives
  about incorrect shlib naming for openmpi* flavours, where the
  shlib guideline is not applicable.
* Mon Jul  4 2022 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 9.4.0:
  * Numerous changes; see here:
    https://www.dealii.org/developer/doxygen/deal.II/changes_between_9_3_3_and_9_4_0.html
* Sat Feb  5 2022 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 9.3.3:
  * Fixes an update flag issue in VectorTools::point_values().
  * Uses portable "cmake -E remove" in quick_tests.
  * Fixes compatibility issues with Boost 1.78.
  * Adds support for the new OneAPI api interface for the Intel
    Threading Building Blocks Library (libtbb).
  * Fixes a compilation issue with sundials.
- Drop dealii-boost-1_78_compat.patch: incorporated upstream.
- Update memory requirement in _constraints to 10 GB and per
  thread memory in %%limit_memory to 2500 MB to avoid OOM errors.
* Wed Jan  5 2022 Atri Bhattacharya <badshah400@gmail.com>
- Add dealii-boost-1_78_compat.patch -- Fix includes to avoid
  build failures against boost 1.77+ (gh#dealii/dealii#13037).
* Wed Nov 10 2021 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 9.3.2:
  * Fix a Sundials/Kinsol issue.
  * Fix a bug for plain_copy in MGTransferMatrixFree.
  * Fix a bug for Checkpointing/Restore of large triangulations
    greater than 4GB.
  * Fix compatibility issues with Intel 18/19 compiler.
  * Fix a compilation issue with step-79 and Intel 18 compiler.
* Fri Jul  2 2021 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 9.3.1:
  * Configuration and compilation fixes for Intel C++ compilers
    versions 18.x and 19.x.
  * Various missing explicit instantitions leading to linker
    errors on Mac OSX platforms.
  * MPI configuration fixes.
  * Fix for a segmentation fault in MappingQGeneric and MappingFE
    at high optimization levels.
* Tue Jun  8 2021 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 9.3.0:
  * Eleven new tutorial programs: Steps 19, 66, 68, 71, 72, 74,
    75, 76, 77, 78, 79 demonstrating solving a variety of
    equations and illustrating functionality in deal.II.
  * Experimental support for simplex and mixed meshes.
  * Improved flexibility of the particle infrastructure.
  * Support for global-coarsening multigrid algorithms.
  * Advances in the matrix-free infrastructure.
  * For full list of changes, see:
    https://www.dealii.org/developer/doxygen/deal.II/changes_between_9_2_0_and_9_3_0.html
- Drop upstreamed patches:
  * dealii-boost-1_75.patch
  * dealii-build-with-muparser-2.3.2.patch.
- Adjust _constraints for slightly increased disk space and
  memory.
- Add openmpi4 as multibuild flavour.
* Fri Apr  9 2021 Guillaume GARDET <guillaume.gardet@opensuse.org>
- Add disk constraints
* Sat Mar 27 2021 Atri Bhattacharya <badshah400@gmail.com>
- _constraints: Increase physical memory to 8 GB to fix slow
  builds causing workers to time out.
* Wed Dec 30 2020 Atri Bhattacharya <badshah400@gmail.com>
- Add dealii-boost-1_75.patch: Fix build with boost 1.75; patch
  taken from upstream git commits (gh#dealii/dealii#11379).
* Mon Aug 17 2020 Michel Normand <normand@linux.vnet.ibm.com>
- for ppc64 (BE) increase memlim from 2k to 3k in spec file
  to avoid build failure in OBS.
* Tue Jul  7 2020 Atri Bhattacharya <badshah400@gmail.com>
- Add dealii-build-with-muparser-2.3.2.patch: Fix muparser 2.3.2
  version detection and thus build (gh#dealii/dealii#10548).
- Drop Group tags.
* Wed Jun  3 2020 Jan Engelhardt <jengelh@inai.de>
- Trim conjecture from description.
- Trim double "library" in summary.
* Sun May 31 2020 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 9.2.0: see
  https://www.dealii.org/developer/doxygen/deal.II/changes_between_9_1_1_and_9_2_0.html
- Drop patches commited upstream:
  * dealii-find-parpack-independently-of-arpack.patch
  * dealii-tbb-version-check.patch
- Set up spec file for enabling python bindings later, when
  gh#dealii/dealii#10423 is fixed; disable building the bindings
  themselves using bcond_with.
- Pass DEAL_II_COMPILE_EXAMPLES=OFF to cmake to prevent compiling
  the examples and generating binaries which are then installed to
  %%_docdir.
- Disable Link-Time-Optimization for aarch64 to prevent builds
  from from failing due to linking taking too long and OBS
  shutting down the worker assuming a stuck job.
- Update _constraints file to ensure enough physical memory in
  build worker to prevent swapping (which slows down and
  eventually times out the build):
  * aarch64: 4 GB (these runs don't time out, so it's okay to have
    a lower memory requirement than for x86_64).
  * x86_64: 7 GB.
- For aarch64, require at least 4000M per thread for compilation.
* Sun May 10 2020 Atri Bhattacharya <badshah400@gmail.com>
- Update to version 9.1.1:
  * Fixed a compatibility bug in GridIn::read_msh with older gmsh
    mesh format files.
- Changes from version 9.1.0: see
  https://www.dealii.org/developer/doxygen/deal.II/changes_between_9_0_1_and_9_1_0.html.
- Changes from version 9.0.0: see
  https://www.dealii.org/developer/doxygen/deal.II/changes_between_8_5_0_and_9_0_0.html.
- Implement multibuild for openmpi2, openmpi3, and serial
  flavours (openmpi1 not supported by dealii).
- Add dealii-tbb-version-check.patch to detect correct version
  of TBB; patch taken from upstream [gh#dealii/dealii#9475].
- Add dealii-find-parpack-independently-of-arpack.patch to detect
  parpack independently of arpack's location; patch taken from
  upstream pull request [gh#dealii/dealii#10197]. Note that *both*
  arpack and parpack are required when building with MPI enabled.
- Use deal_II*-devel as the devel pkg name to be consistent with
  shared lib packaging policy; add Provides: dealii*-devel to help
  with upgrade.
- Use ninja for build (BuildRequires: ninja).
- Use memory_constraints to require per-thread memory >= 2 GB;
  introduces BuildRequires: memory-constraints.
- Use %%cmake_build instead of running make.
- Update cmake options in keeping with upstream changes.
- Add _constraints file to limit builds to at least 2 GB memory,
  anything less causes out-of-memory problems.
- Update License tag: dealii is now licensed as LGPL-2.1-or-later.
- Remove hashbangs from non-executable files.
- Link duplicate files in %%{_docdir}.
- Disable builds on i586 as linking takes too long leading to OBS
  worker assuming a "stuck job" and killing the VM.
- Disable building documentation for openSUSE 15.1 as it fails.
* Tue Nov 21 2017 junghans@votca.org
- Update to version 8.5.1: Only build fixes
  https://github.com/dealii/dealii/compare/v8.5.0...v8.5.1
- Drop dealii-boost-1_64.patch, got merged upstream
* Wed Aug  9 2017 badshah400@gmail.com
- Update to version 8.5.0: See
  http://www.dealii.org/8.5.0/doxygen/deal.II/changes_between_8_4_2_and_8_5_0.html
  for the list of changes (too many to list here).
- Add dealii-boost-1_64.patch: Fix compilation against boost 1.64;
  patch taken from upstream git (gh#dealii/dealii#4309).
- Move documentation files installed to non-standard locations to
  standard docdir.
- Package cmake files and data files in -devel package.
* Mon Jan 18 2016 kkhere.geo@gmail.com
- add missing library file libtbb.so* to rpm, otherwise we get
  missing dependency when trying to install
* Sun Feb 24 2013 scorot@free.fr
- add patch dealii-7.2.0-no-rpath.patch
- fix files list
- change %%%%libname according to so name
- fix build requirements for SLE 11 and 12.1
- disable %%%%{?_smp_mflags} with make because build process
  requires to much memory
* Sun Feb 10 2013 scorot@free.fr
- first package
